local myNAME = "merUndefinedGlobalsCatcher"
local frexp = math.frexp
local strfind = string.find
local strformat = string.format
local tabinsert = table.insert

local reported = setmetatable({}, {__index = function() return 0 end})
local msgwin = nil


local function globalmiss(tab, key)
    if type(key) == nil then
        return
    end

    local n = reported[key] + 1
    local p = 10
    reported[key] = n

    if frexp(n) ~= 0.5 then
        -- n is not a power of 2
        return
    end

    if type(key) == "string" then
        if n == 1 and strfind(key, "^ZO_") then
            -- don't report the very first lookup of ZO object,
            -- it's probably going to be created
            return
        end
        msgwin:AddText(strformat("%3dx %q", n, key))
    else
        msgwin:AddText(strformat("%3dx %s", n, tostring(key)))
    end
end


local function onLoad(eventCode, addOnName)
    if addOnName ~= myNAME then return end
    EVENT_MANAGER:UnregisterForEvent(myNAME, eventCode)

    local libmw = LibStub:GetLibrary("LibMsgWin-1.0")
    msgwin = libmw:CreateMsgWindow("merUndefinedGlobalsCatcherWindow",
                                   "undefined globals")
    msgwin:SetDimensions(700, 400)
    msgwin:SetHidden(true)

    SLASH_COMMANDS["/undefs"] = function() msgwin:ToggleHidden() end

    setmetatable(_G, {__index = globalmiss})
end


EVENT_MANAGER:RegisterForEvent(myNAME, EVENT_ADD_ON_LOADED, onLoad)
